"use strict";
import {
  aTimeout,
  elementUpdated,
  expect,
  nextFrame,
  oneEvent
} from "@open-wc/testing";
import { sendKeys } from "@web/test-runner-commands";
import { html } from "@spectrum-web-components/base";
import { spy } from "sinon";
import { a11ySnapshot, findAccessibilityNode } from "@web/test-runner-commands";
import fieldDefaults, {
  m as field
} from "../stories/split-button-accent-field.stories.js";
import moreDefaults, {
  m as more
} from "../stories/split-button-accent-more.stories.js";
import { sendMouse } from "../../../test/plugins/browser.js";
import { fixture } from "../../../test/testing-helpers.js";
export function runSplitButtonTests(wrapInDiv, deprecatedMenu) {
  it('loads [type="field"] splitbutton accessibly', async () => {
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    await nextFrame();
    await nextFrame();
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    await expect(el1).to.be.accessible();
    await expect(el2).to.be.accessible();
  });
  it('loads [type="field"] splitbutton accessibly with deprecated syntax', async () => {
    const test = await fixture(html`
            <div>
                <sp-split-button>${deprecatedMenu()}</sp-split-button>
                <sp-split-button left>${deprecatedMenu()}</sp-split-button>
            </div>
        `);
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    await nextFrame();
    await nextFrame();
    await expect(el1).to.be.accessible();
    await expect(el2).to.be.accessible();
  });
  it('loads [type="more"] splitbutton accessibly', async () => {
    const test = await fixture(
      wrapInDiv(more({ ...moreDefaults.args, ...more.args }))
    );
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    await nextFrame();
    await nextFrame();
    await expect(el1).to.be.accessible();
    await expect(el2).to.be.accessible();
  });
  it('loads [type="more"] splitbutton accessibly with deprecated syntax', async () => {
    const test = await fixture(html`
            <div>
                <sp-split-button type="more">
                    ${deprecatedMenu()}
                </sp-split-button>
                <sp-split-button type="more" left>
                    ${deprecatedMenu()}
                </sp-split-button>
            </div>
        `);
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    await nextFrame();
    await nextFrame();
    await expect(el1).to.be.accessible();
    await expect(el2).to.be.accessible();
  });
  it("loads splitbutton accessibly and checks labels", async function() {
    const test = await fixture(html`
            <div>
                <sp-split-button>${deprecatedMenu()}</sp-split-button>
                <sp-split-button label="Test" left>
                    ${deprecatedMenu()}
                </sp-split-button>
            </div>
        `);
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    await nextFrame();
    await nextFrame();
    const snapshot = await a11ySnapshot(
      {}
    );
    expect(
      findAccessibilityNode(
        snapshot,
        (node) => (node.role === "button" || node.role === "buttonmenu") && node.name === "Option 1" && node.haspopup
      ),
      'Has a named "button" element with haspopup="true" and name="Option 1"'
    ).to.not.be.null;
    expect(
      findAccessibilityNode(
        snapshot,
        (node) => node.role === "button" && node.name === "Option 1" && !node.haspopup
      ),
      'Has a named "button" element with haspopup="false" and name="Option 1"'
    ).to.not.be.null;
    expect(
      findAccessibilityNode(
        snapshot,
        (node) => (node.role === "button" || node.role === "buttonmenu") && node.name === "Test" && node.haspopup
      ),
      'Has a named "button" element with haspopup="true" and name="Test"'
    ).to.not.be.null;
    expect(
      findAccessibilityNode(
        snapshot,
        (node) => node.role === "button" && node.name === "Test" && !node.haspopup
      ),
      'Has a named "button" element with haspopup="false" and name="Test"'
    ).to.not.be.null;
  });
  it('[type="field"] toggles open/close multiple time', async () => {
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const trigger = el.shadowRoot.querySelector(".trigger");
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
    let opened = oneEvent(el, "sp-opened");
    el.open = true;
    await opened;
    expect(el.open).to.be.true;
    expect(trigger).to.have.attribute("aria-expanded", "true");
    expect(trigger).to.have.attribute("aria-controls", "menu");
    let closed = oneEvent(el, "sp-closed");
    el.open = false;
    await closed;
    expect(el.open).to.be.false;
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
    opened = oneEvent(el, "sp-opened");
    el.open = true;
    await opened;
    expect(el.open).to.be.true;
    expect(trigger).to.have.attribute("aria-expanded", "true");
    expect(trigger).to.have.attribute("aria-controls", "menu");
    closed = oneEvent(el, "sp-closed");
    el.open = false;
    await closed;
    expect(el.open).to.be.false;
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
  });
  it('[type="field"] opens, then closes, on subsequent clicks', async () => {
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const { trigger } = el;
    const rect = trigger.getBoundingClientRect();
    const open = oneEvent(el, "sp-opened");
    sendMouse({
      steps: [
        {
          position: [
            rect.left + rect.width / 2,
            rect.top + rect.height / 2
          ],
          type: "click"
        }
      ]
    });
    await open;
    expect(el.open).to.be.true;
    await aTimeout(50);
    expect(el.open).to.be.true;
    const close = oneEvent(el, "sp-closed");
    sendMouse({
      steps: [
        {
          position: [
            rect.left + rect.width / 2,
            rect.top + rect.height / 2
          ],
          type: "click"
        }
      ]
    });
    await close;
    expect(el.open).to.be.false;
    await aTimeout(50);
    expect(el.open).to.be.false;
  });
  it('[type="field"] opens and selects in a single pointer button interaction', async () => {
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const thirdItem = el.querySelector(
      "sp-menu-item:nth-of-type(3)"
    );
    const trigger = el.shadowRoot.querySelector(".trigger");
    const boundingRect = trigger.getBoundingClientRect();
    expect(el.value).to.not.equal(thirdItem.value);
    const opened = oneEvent(el, "sp-opened");
    await sendMouse({
      steps: [
        {
          type: "move",
          position: [
            boundingRect.x + boundingRect.width / 2,
            boundingRect.y + boundingRect.height / 2
          ]
        },
        {
          type: "down"
        }
      ]
    });
    await opened;
    const thirdItemRect = thirdItem.getBoundingClientRect();
    const closed = oneEvent(el, "sp-closed");
    await sendMouse({
      steps: [
        {
          type: "move",
          position: [
            thirdItemRect.x + thirdItemRect.width / 2,
            thirdItemRect.y + thirdItemRect.height / 2
          ]
        },
        {
          type: "up"
        }
      ]
    });
    await closed;
    expect(el.open).to.be.false;
    expect(el.value).to.equal(thirdItem.value);
  });
  it('[type="more"] toggles open/close multiple time', async () => {
    const test = await fixture(
      wrapInDiv(more({ ...moreDefaults.args, ...more.args }))
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const trigger = el.shadowRoot.querySelector(".trigger");
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
    let opened = oneEvent(el, "sp-opened");
    el.open = true;
    await opened;
    expect(el.open).to.be.true;
    expect(trigger).to.have.attribute("aria-expanded", "true");
    expect(trigger).to.have.attribute("aria-controls", "menu");
    let closed = oneEvent(el, "sp-closed");
    el.open = false;
    await closed;
    expect(el.open).to.be.false;
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
    opened = oneEvent(el, "sp-opened");
    el.open = true;
    await opened;
    expect(el.open).to.be.true;
    expect(trigger).to.have.attribute("aria-expanded", "true");
    expect(trigger).to.have.attribute("aria-controls", "menu");
    closed = oneEvent(el, "sp-closed");
    el.open = false;
    await closed;
    expect(el.open).to.be.false;
    expect(trigger).to.have.attribute("aria-expanded", "false");
    expect(trigger).not.to.have.attribute("aria-controls");
  });
  it('[type="more"] opens and selects in a single pointer button interaction', async () => {
    const thirdItemSpy = spy();
    const test = await fixture(
      wrapInDiv(
        more({
          ...moreDefaults.args,
          ...more.args,
          thirdItemHandler: () => thirdItemSpy()
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const thirdItem = el.querySelector(
      "sp-menu-item:nth-of-type(3)"
    );
    const trigger = el.shadowRoot.querySelector(".trigger");
    const boundingRect = trigger.getBoundingClientRect();
    expect(el.value).to.not.equal(thirdItem.value);
    const opened = oneEvent(el, "sp-opened");
    await sendMouse({
      steps: [
        {
          type: "move",
          position: [
            boundingRect.x + boundingRect.width / 2,
            boundingRect.y + boundingRect.height / 2
          ]
        },
        {
          type: "down"
        }
      ]
    });
    await opened;
    const thirdItemRect = thirdItem.getBoundingClientRect();
    const closed = oneEvent(el, "sp-closed");
    await sendMouse({
      steps: [
        {
          type: "move",
          position: [
            thirdItemRect.x + thirdItemRect.width / 2,
            thirdItemRect.y + thirdItemRect.height / 2
          ]
        },
        {
          type: "up"
        }
      ]
    });
    await closed;
    expect(el.open).to.be.false;
    expect(thirdItemSpy.callCount).to.equal(1);
  });
  it('[type="more"] opens, then closes, on subsequent clicks', async () => {
    const test = await fixture(
      wrapInDiv(more({ ...moreDefaults.args, ...more.args }))
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    const { trigger } = el;
    const rect = trigger.getBoundingClientRect();
    const open = oneEvent(el, "sp-opened");
    sendMouse({
      steps: [
        {
          position: [
            rect.left + rect.width / 2,
            rect.top + rect.height / 2
          ],
          type: "click"
        }
      ]
    });
    await open;
    expect(el.open).to.be.true;
    await aTimeout(50);
    expect(el.open).to.be.true;
    const close = oneEvent(el, "sp-closed");
    sendMouse({
      steps: [
        {
          position: [
            rect.left + rect.width / 2,
            rect.top + rect.height / 2
          ],
          type: "click"
        }
      ]
    });
    await close;
    expect(el.open).to.be.false;
    await aTimeout(50);
    expect(el.open).to.be.false;
  });
  it('receives "focus()"', async () => {
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    const el1 = test.querySelector("sp-split-button");
    const el2 = test.querySelector("sp-split-button[left]");
    await elementUpdated(el1);
    await elementUpdated(el2);
    el1.focus();
    await elementUpdated(el1);
    expect(document.activeElement === el1).to.be.true;
    expect(el1.shadowRoot.activeElement).to.equal(el1.focusElement);
    el2.focus();
    await elementUpdated(el2);
    expect(document.activeElement === el2).to.be.true;
    expect(el2.shadowRoot.activeElement === el2.focusElement).to.be.true;
  });
  it('[type="field"] manages `selectedItem`', async () => {
    var _a, _b;
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    expect((_a = el.selectedItem) == null ? void 0 : _a.itemText).to.equal("Option 1");
    expect(el.open).to.be.false;
    const item3 = el.querySelector("sp-menu-item:nth-child(3)");
    const opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    await elementUpdated(el);
    expect(el.open).to.be.true;
    const closed = oneEvent(el, "sp-closed");
    item3.click();
    await closed;
    await elementUpdated(el);
    expect((_b = el.selectedItem) == null ? void 0 : _b.itemText).to.equal("Short");
    expect(el.open).to.be.false;
    expect(document.activeElement === el).to.be.true;
    expect(el.shadowRoot.activeElement === el.focusElement).to.be.true;
  });
  it('[type="more"] manages `selectedItem.itemText`', async () => {
    var _a, _b;
    const test = await fixture(
      wrapInDiv(more({ ...moreDefaults.args, ...more.args }))
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    expect((_a = el.selectedItem) == null ? void 0 : _a.itemText).to.equal("Option 1");
    expect(el.open).to.be.false;
    const item3 = el.querySelector("sp-menu-item:nth-child(3)");
    const opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    expect(el.open).to.be.true;
    const closed = oneEvent(el, "sp-closed");
    item3.click();
    await closed;
    expect(el.open).to.be.false;
    expect((_b = el.selectedItem) == null ? void 0 : _b.itemText).to.equal("Option 1");
  });
  it('passes click events as [type="field"]', async () => {
    var _a, _b, _c, _d;
    const firstItemSpy = spy();
    const secondItemSpy = spy();
    const thirdItemSpy = spy();
    const test = await fixture(
      wrapInDiv(
        field({
          ...fieldDefaults.args,
          ...field.args,
          firstItemHandler: () => firstItemSpy(),
          secondItemHandler: () => secondItemSpy(),
          thirdItemHandler: () => thirdItemSpy()
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    expect((_a = el.selectedItem) == null ? void 0 : _a.itemText).to.equal("Option 1");
    expect(el.open).to.be.false;
    const item1 = el.querySelector("sp-menu-item:nth-child(1)");
    const item2 = el.querySelector("sp-menu-item:nth-child(2)");
    const item3 = el.querySelector("sp-menu-item:nth-child(3)");
    const main = el.button;
    main.click();
    await elementUpdated(el);
    expect(firstItemSpy.called, "first called").to.be.true;
    expect(firstItemSpy.calledOnce, "first calledOnce").to.be.true;
    let opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    await elementUpdated(el);
    expect(el.open, "open").to.be.true;
    let closed = oneEvent(el, "sp-closed");
    item3.click();
    await closed;
    await nextFrame();
    await elementUpdated(el);
    expect(el.open, "not open").to.be.false;
    expect(thirdItemSpy.called, "third called").to.be.true;
    expect(thirdItemSpy.calledOnce, "third calledOnce").to.be.true;
    main.click();
    await elementUpdated(el);
    expect(el.open).to.be.false;
    expect((_b = el.selectedItem) == null ? void 0 : _b.itemText).to.equal("Short");
    expect(thirdItemSpy.called, "third called, still").to.be.true;
    expect(thirdItemSpy.callCount, "third callCount").to.equal(2);
    expect(thirdItemSpy.calledTwice, "third calledTwice").to.be.true;
    await sendKeys({
      press: "Tab"
    });
    opened = oneEvent(el, "sp-opened");
    await sendKeys({
      press: "k"
    });
    sendKeys({
      press: "ArrowDown"
    });
    await opened;
    await elementUpdated(el);
    expect(el.open, "reopened").to.be.true;
    closed = oneEvent(el, "sp-closed");
    item2.click();
    await closed;
    await nextFrame();
    await elementUpdated(el);
    main.click();
    await elementUpdated(el);
    expect(el.open).to.be.false;
    expect((_c = el.selectedItem) == null ? void 0 : _c.itemText).to.equal("Option Really Extended");
    expect(secondItemSpy.called, "second called").to.be.true;
    expect(secondItemSpy.calledTwice, "second twice").to.be.true;
    opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    await elementUpdated(el);
    expect(el.open, "opened again").to.be.true;
    closed = oneEvent(el, "sp-closed");
    item1.click();
    await closed;
    await nextFrame();
    await elementUpdated(el);
    main.click();
    await elementUpdated(el);
    expect((_d = el.selectedItem) == null ? void 0 : _d.itemText).to.equal("Option 1");
    expect(firstItemSpy.called, "first called, sill").to.be.true;
    expect(firstItemSpy.callCount, "first callCount").to.equal(3);
  });
  it('passes click events as [type="more"]', async () => {
    var _a, _b, _c;
    const firstItemSpy = spy();
    const secondItemSpy = spy();
    const thirdItemSpy = spy();
    const test = await fixture(
      wrapInDiv(
        more({
          ...moreDefaults.args,
          ...more.args,
          firstItemHandler: () => firstItemSpy(),
          secondItemHandler: () => secondItemSpy(),
          thirdItemHandler: () => thirdItemSpy()
        })
      )
    );
    const el = test.querySelector("sp-split-button");
    await elementUpdated(el);
    await nextFrame();
    await nextFrame();
    expect((_a = el.selectedItem) == null ? void 0 : _a.itemText).to.equal("Option 1");
    expect(el.open).to.be.false;
    const item2 = el.querySelector("sp-menu-item:nth-child(2)");
    const item3 = el.querySelector("sp-menu-item:nth-child(3)");
    const main = el.shadowRoot.querySelector(
      "#button"
    );
    main.click();
    expect(el.open).to.be.false;
    await elementUpdated(el);
    await nextFrame();
    expect(firstItemSpy.called, "1st called").to.be.true;
    expect(firstItemSpy.calledOnce, "1st called once").to.be.true;
    expect(el.open).to.be.false;
    let opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    await elementUpdated(el);
    expect(el.open).to.be.true;
    let closed = oneEvent(el, "sp-closed");
    item3.click();
    await closed;
    await nextFrame();
    await elementUpdated(el);
    expect(el.open, "not open").to.be.false;
    expect((_b = el.selectedItem) == null ? void 0 : _b.itemText).to.equal("Option 1");
    expect(thirdItemSpy.called, "3rd called").to.be.true;
    expect(thirdItemSpy.calledOnce, "3rd called once").to.be.true;
    opened = oneEvent(el, "sp-opened");
    el.click();
    await opened;
    await elementUpdated(el);
    expect(el.open).to.be.true;
    closed = oneEvent(el, "sp-closed");
    item2.click();
    await closed;
    await nextFrame();
    await nextFrame();
    await elementUpdated(el);
    expect(el.open).to.be.false;
    expect((_c = el.selectedItem) == null ? void 0 : _c.itemText).to.equal("Option 1");
    expect(secondItemSpy.called, "2nd called").to.be.true;
    expect(secondItemSpy.calledOnce, "2nd called once").to.be.true;
    main.click();
    await elementUpdated(el);
    expect(firstItemSpy.called).to.be.true;
    expect(firstItemSpy.calledTwice, "1st called twice").to.be.true;
  });
}
//# sourceMappingURL=index.js.map
